import requests
from alquran.config import API_SURAH_URL


def get_surah_list():
    """Mengambil daftar surah dari API."""
    response = requests.get(f"{API_SURAH_URL}/surat")
    return response.json()["data"]


def get_surah_details(nomor):
    """Mengambil detail surah dari API berdasarkan nomor surah."""
    response = requests.get(f"{API_SURAH_URL}/surat/{nomor}")
    return response.json()["data"]


def get_tafsir_details(nomor):
    """Mengambil detail tafsir dari API berdasarkan nomor surah."""
    response = requests.get(f"{API_SURAH_URL}/tafsir/{nomor}")
    return response.json()["data"]


def daftar_surat():
    """Menampilkan daftar surah dalam Al-Quran."""
    surahs = get_surah_list()
    return surahs


def detail_surat(nomor_surah):
    """Menampilkan detail surah berdasarkan nomor surah."""
    if nomor_surah > 114:
        print("Al-Quran hanya berisi sebanyak 114 surah")
        return False
    surah = get_surah_details(nomor_surah)
    return surah


def isi_surat(nomor_surah, ayat=None):
    """Memberikan isi bacaan ayat ayat berdasarkan nomor surah"""
    if nomor_surah > 114:
        print("Al-Quran hanya berisi sebanyak 114 surah")
        return False
    surah = get_surah_details(nomor_surah)
    ayat_ayat = surah["ayat"]

    # Jika ada opsi ayat, filter ayat yang sesuai
    if ayat is not None:
        ayat_filter = []
        for bagian in ayat.split(","):
            if "-" in bagian:
                start, end = bagian.split("-")
                ayat_filter.extend(range(int(start), int(end) + 1))
            else:
                ayat_filter.append(int(bagian))
        ayat_ayat = [
            ayat for ayat in ayat_ayat if int(ayat["nomorAyat"]) in ayat_filter
        ]
    if not ayat_ayat:
        print("Ayat tidak tersedia seperti filter")
        return False
    return (surah, ayat_ayat)


def tafsir_surat(nomor_surah):
    """Menampilkan tafsir untuk surah berdasarkan nomor surah."""
    if nomor_surah > 114:
        print("Al-Quran hanya berisi sebanyak 114 surah")
        return False
    tafsir_data = get_tafsir_details(nomor_surah)
    return tafsir_data



import requests
import re
from alquran.config import API_HADITH_URL, RANGE_PATTERN


def get_hadits_list():
    """Mengambil daftar buku hadits dari API"""
    response = requests.get(f"{API_HADITH_URL}/books")
    return response.json()


def get_hadits(id_hadits, nomor_hadits):
    """Mengambil riwayat hadits sesuai dengan id nya (tirmidzi, muslim, dsb)"""
    response = requests.get(f"{API_HADITH_URL}/books/{id_hadits}/{nomor_hadits}")
    return response.json()


def get_hadits_range(id_hadits, range_nomor):
    """Mengambil riwayat hadits sesuai dengan id dan berdasarkan range nya"""
    response = requests.get(f"{API_HADITH_URL}/books/{id_hadits}?range={range_nomor}")
    return response.json()


def daftar_hadith():
    """Menampilkan daftar riwayat-riwayat hadits"""
    hadits = get_hadits_list()
    data = hadits["data"]
    if hadits["code"] != 200 or hadits["error"] is True:
        print(hadits["message"])
        return False
    return data


def lihat_hadith(id_hadits, nomor_hadits, range=None):
    """Memberikan isi dari sebuah hadits berdasarkan spesifikasi"""
    if range:
        range = re.findall(RANGE_PATTERN, range)
        if len(range) == 0:
            print("Invalid range pattern")
            return False

        hadits = get_hadits_range(id_hadits, range[0])
        if hadits["code"] == 400 and hadits["error"] is True:
            print("Tidak bisa >300 range (Masalah performa)")
            return False
        if hadits["code"] == 404 and hadits["error"] is True:
            print(hadits["message"])
            return False
        data = hadits["data"]
        return data
    else:
        if not nomor_hadits:
            print("Nomor hadits tidak boleh kosong kecuali range diberikan")
            return False
        hadits = get_hadits(id_hadits, nomor_hadits)
        if hadits["code"] == 404 and hadits["error"] is True:
            print(hadits["message"])
            return False
        data = hadits["data"]
        return data



import click
import shutil
from alquran.core import (
    lihat_hadith,
    isi_surat,
    detail_surat,
    tafsir_surat,
    daftar_surat,
)


@click.group()
def cli():
    """CLI Al-Quran."""
    pass


@cli.group()
def surah():
    """Kategori : Surah - Surah Al-Qur'an"""
    pass


@cli.group()
def hadits():
    """Kategori : Riwayat - Riwayat Hadits"""
    pass


@hadits.command()
def daftar_hadits():
    """Menampilkan daftar riwayat-riwayat hadits"""
    data = daftar_hadits()
    if data is False:
        return
    for idx, riwayat in enumerate(data, start=1):
        click.echo(
            f"{idx}. {riwayat['name']} | 1 - {riwayat['available']} ({riwayat['id']})"
        )


@hadits.command()
@click.argument("id_hadits")
@click.argument("nomor_hadits", type=int)
@click.option(
    "--range",
    default=None,
    help="Range dari beberapa hadits tertentu, contoh --range 1-3",
)
def lihat_hadits(id_hadits, nomor_hadits, range):
    """Memberikan isi dari sebuah hadits berdasarkan spesifikasi"""
    data = lihat_hadith(id_hadits, nomor_hadits, range)
    if data is False:
        return
    if range:
        click.echo("-" * shutil.get_terminal_size().columns + "\n")
        for hadith in data["hadiths"]:
            click.echo(f"“{hadith['arab']}”\n")
            click.echo(f" “{hadith['id']}” - {data['name']} ({hadith['number']})\n")
            click.echo("-" * shutil.get_terminal_size().columns + "\n")
    else:
        click.echo("-" * shutil.get_terminal_size().columns + "\n")
        click.echo(f"“{data['contents']['arab']}”\n")
        click.echo(
            f" “{data['contents']['id']}” - {data['name']} ({data['contents']['number']})"
        )
        click.echo("\n" + "-" * shutil.get_terminal_size().columns)


@surah.command()
def daftar_surah():
    """Menampilkan daftar surah dalam Al-Quran"""
    surahs = daftar_surat()
    if surahs is False:
        return
    for surah in surahs:
        print(
            f"{surah['nomor']}. {surah['nama']} ({surah['namaLatin']}) - {surah['jumlahAyat']} Ayat"
        )


@surah.command()
@click.argument("nomor_surah", type=int)
def detail_surah(nomor_surah):
    """Menampilkan detail surah berdasarkan nomor surah"""
    surah = detail_surat(nomor_surah)
    if surah is False:
        return
    click.echo(f"Surah: {surah['nama']} ({surah['namaLatin']})")
    click.echo(f"Jumlah Ayat: {surah['jumlahAyat']}")
    click.echo(f"Tempat Turun: {surah['tempatTurun']}")
    click.echo(f"Arti: {surah['arti']}")
    click.echo(
        f"Deskripsi: {surah['deskripsi'].replace('<i>', '').replace('</i>', '').replace('<br>', '').replace('</br>', '')}"
    )
    # click.echo("\nAudio:")
    # for key, audio in surah['audioFull'].items():
    #     click.echo(f"  Reciter {key}: {audio}")


@surah.command()
@click.argument("nomor_surah", type=int)
@click.option(
    "--ayat",
    default=None,
    help="Filter ayat tertentu sesuai rentang, contoh --ayat 5 / --ayat 2-5",
)
def isi_surah(nomor_surah, ayat):
    """Memberikan isi bacaan ayat ayat berdasarkan nomor surah"""
    res = isi_surat(nomor_surah, ayat)
    if res is False:
        return
    surah = res[0]
    ayat_ayat = res[1]
    click.echo(
        f"{surah['namaLatin']} {surah['nomor']}:1-{surah['jumlahAyat']} ({surah['nama']})\n"
    )
    for ayah in ayat_ayat:
        click.echo(f" “{ayah['teksArab']}”")
        click.echo(f"{ayah['teksLatin']} ({ayah['nomorAyat']})\n")
    click.echo("Terjemahan : ")
    for ayah in ayat_ayat:
        click.echo(f"{ayah['nomorAyat']}. {ayah['teksIndonesia']}")
        click.echo("-" * shutil.get_terminal_size().columns)


@surah.command()
@click.argument("nomor_surah", type=int)
def tafsir_surah(nomor_surah):
    """Menampilkan tafsir untuk surah berdasarkan nomor surah"""
    tafsir_data = tafsir_surat(nomor_surah)
    if tafsir_data is False:
        return
    click.echo(
        f"Tafsir untuk Surah {tafsir_data['nama']} ({tafsir_data['namaLatin']}):"
    )
    for tafsir in tafsir_data["tafsir"]:
        click.echo(f"Ayat {tafsir['ayat']} - {tafsir['teks']}")



import importlib.metadata
import httpx


def check_package_update(package_name):
    try:
        # Get installed package version
        installed_version = importlib.metadata.version(package_name)

        # Get latest version from PyPI
        response = httpx.get(f"https://pypi.org/pypi/{package_name}/json")
        response.raise_for_status()
        latest_version = response.json()["info"]["version"]

        if installed_version != latest_version:
            print(
                f"Package '{package_name}' is outdated: {installed_version} -> {latest_version}"
            )
            print(
                f"Please immediately update using 'pip install --upgrade {package_name}'"
            )
    except importlib.metadata.PackageNotFoundError:
        print(f"Package '{package_name}' is not installed.")
    except httpx.RequestError as e:
        print(f"Error checking latest version: {e}")



from alquran.hadits import (
    get_hadits,
    get_hadits_list,
    get_hadits_range,
    daftar_hadith,
    lihat_hadith,
)
from alquran.surah import (
    get_surah_details,
    get_surah_list,
    get_tafsir_details,
    daftar_surat,
    isi_surat,
    detail_surat,
    tafsir_surat,
)
from alquran.config import API_HADITH_URL, API_SURAH_URL, RANGE_PATTERN
from alquran.utils.version import check_package_update

check_package_update("alquran-cli")
